@@ -1,4 +1,4 @@
-/*  pl-wic.c,v 1.31 1995/06/14 08:22:16 jan Exp
+/*  pl-wic.c,v 1.32 1995/07/24 15:08:34 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -7,7 +7,7 @@
     Purpose: load and save intermediate code files
 */
 
-#define O_DEBUG 1
+/*#define O_DEBUG 1*/
 #include "pl-incl.h"
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
